@@ -18,9 +18,19 @@
 #' initial groups. Will be evaluated in the context of the node data.
 #' @param type The type of component to find. Either `'weak'` or `'strong'`
 #' @param directed Should direction of edges be used for the calculations
+#' @param n_groups Integer scalar, the desired number of communities. If too low
+#' or two high, then an error message is given. The measure is applied to the
+#' full graph so the number of groups returned may be lower for focused graphs
 #' @param trials Number of times partition of the network should be attempted
 #' @param steps The number of steps in the random walks
 #' @param options Settings passed on to `igraph::arpack()`
+#' @param resolution Resolution of the modularity function used internally in
+#' the algorithm
+#' @param objective_function Either `"CPM"` (constant potts model) or
+#' `"modularity"`. Sets the objective function to use.
+#' @param beta Parameter affecting the randomness in the Leiden algorithm. This
+#' affects only the refinement step of the algorithm.
+#' @param n The number of iterations to run the clustering
 #' @param ... arguments passed on to [igraph::cluster_spinglass()]
 #'
 #' @return a numeric vector with the membership for each node in the graph. The
@@ -42,29 +52,39 @@ NULL
 #' @export
 group_components <- function(type = 'weak') {
   expect_nodes()
-  group <- as.integer(components(graph = .G(), mode = type)$membership)
+  group <- as.integer(components(graph = .G(), mode = type)$membership[focus_ind(.G(), 'nodes')])
   desc_enumeration(group)
 }
 #' @describeIn group_graph Group densely connected nodes using [igraph::cluster_edge_betweenness()]
-#' @importFrom igraph membership cluster_edge_betweenness
+#' @importFrom igraph membership cluster_edge_betweenness cut_at
 #' @export
-group_edge_betweenness <- function(weights = NULL, directed = TRUE) {
+group_edge_betweenness <- function(weights = NULL, directed = TRUE, n_groups = NULL) {
   expect_nodes()
   weights <- enquo(weights)
-  weights <- eval_tidy(weights, .E())
-  # NULL in weights is for once respected despite a weight attribute
-  group <- as.integer(membership(cluster_edge_betweenness(graph = .G(), weights = weights, directed = directed)))
+  weights <- eval_tidy(weights, .E()) %||% NA
+  group <- cluster_edge_betweenness(graph = .G(), weights = weights, directed = directed)
+  if (is.null(n_groups)) {
+    group <- membership(group)
+  } else {
+    group <- cut_at(group, no = n_groups)
+  }
+  group <- as.integer(group[focus_ind(.G(), 'nodes')])
   desc_enumeration(group)
 }
 #' @describeIn group_graph Group nodes by optimising modularity using [igraph::cluster_fast_greedy()]
-#' @importFrom igraph membership cluster_fast_greedy
+#' @importFrom igraph membership cluster_fast_greedy cut_at
 #' @export
-group_fast_greedy <- function(weights = NULL) {
+group_fast_greedy <- function(weights = NULL, n_groups = NULL) {
   expect_nodes()
   weights <- enquo(weights)
-  weights <- eval_tidy(weights, .E())
-  # NULL in weights is for once respected despite a weight attribute
-  group <- as.integer(membership(cluster_fast_greedy(graph = .G(), weights = weights)))
+  weights <- eval_tidy(weights, .E()) %||% NA
+  group <- cluster_fast_greedy(graph = .G(), weights = weights)
+  if (is.null(n_groups)) {
+    group <- membership(group)
+  } else {
+    group <- cut_at(group, no = n_groups)
+  }
+  group <- as.integer(group[focus_ind(.G(), 'nodes')])
   desc_enumeration(group)
 }
 #' @describeIn group_graph Group nodes by minimizing description length using [igraph::cluster_infomap()]
@@ -73,13 +93,10 @@ group_fast_greedy <- function(weights = NULL) {
 group_infomap <- function(weights = NULL, node_weights = NULL, trials = 10) {
   expect_nodes()
   weights <- enquo(weights)
-  weights <- eval_tidy(weights, .E())
-  if (is.null(weights)) {
-    weights <- NA
-  }
+  weights <- eval_tidy(weights, .E()) %||% NA
   node_weights <- enquo(node_weights)
   node_weights <- eval_tidy(node_weights, .N())
-  group <- as.integer(membership(cluster_infomap(graph = .G(), e.weights = weights, v.weights = node_weights, nb.trials = trials)))
+  group <- as.integer(membership(cluster_infomap(graph = .G(), e.weights = weights, v.weights = node_weights, nb.trials = trials))[focus_ind(.G(), 'nodes')])
   desc_enumeration(group)
 }
 #' @describeIn group_graph Group nodes by propagating labels using [igraph::cluster_label_prop()]
@@ -88,44 +105,75 @@ group_infomap <- function(weights = NULL, node_weights = NULL, trials = 10) {
 group_label_prop <- function(weights = NULL, label = NULL, fixed = NULL) {
   expect_nodes()
   weights <- enquo(weights)
-  weights <- eval_tidy(weights, .E())
-  if (is.null(weights)) {
-    weights <- NA
-  }
+  weights <- eval_tidy(weights, .E()) %||% NA
   N <- .N()
   label <- enquo(label)
   label <- eval_tidy(label, N)
   fixed <- enquo(fixed)
   fixed <- eval_tidy(fixed, N)
-  group <- as.integer(membership(cluster_label_prop(graph = .G(), weights = weights, initial = label, fixed = fixed)))
+  group <- as.integer(membership(cluster_label_prop(graph = .G(), weights = weights, initial = label, fixed = fixed))[focus_ind(.G(), 'nodes')])
   desc_enumeration(group)
 }
 #' @describeIn group_graph Group nodes based on the leading eigenvector of the modularity matrix using [igraph::cluster_leading_eigen()]
-#' @importFrom igraph membership cluster_leading_eigen
+#' @importFrom igraph membership cluster_leading_eigen cut_at
 #' @export
-group_leading_eigen <- function(weights = NULL, steps = -1, label = NULL, options = igraph::arpack_defaults) {
+group_leading_eigen <- function(weights = NULL, steps = -1, label = NULL, options = igraph::arpack_defaults, n_groups = NULL) {
   expect_nodes()
   weights <- enquo(weights)
-  weights <- eval_tidy(weights, .E())
-  if (is.null(weights)) {
-    weights <- NA
-  }
+  weights <- eval_tidy(weights, .E()) %||% NA
   label <- enquo(label)
   label <- eval_tidy(label, .N())
-  group <- as.integer(membership(cluster_leading_eigen(graph = .G(), steps = steps, weights = weights, start = label, options = options)))
+  group <- cluster_leading_eigen(graph = .G(), steps = steps, weights = weights, start = label, options = options)
+  if (is.null(n_groups)) {
+    group <- membership(group)
+  } else {
+    group <- cut_at(group, no = n_groups)
+  }
+  group <- as.integer(group[focus_ind(.G(), 'nodes')])
   desc_enumeration(group)
 }
 #' @describeIn group_graph Group nodes by multilevel optimisation of modularity using [igraph::cluster_louvain()]
 #' @importFrom igraph membership cluster_louvain
 #' @export
-group_louvain <- function(weights = NULL) {
+group_louvain <- function(weights = NULL, resolution = 1) {
   expect_nodes()
   weights <- enquo(weights)
-  weights <- eval_tidy(weights, .E())
-  if (is.null(weights)) {
-    weights <- NA
+  weights <- eval_tidy(weights, .E()) %||% NA
+  group <- as.integer(membership(cluster_louvain(graph = .G(), weights = weights, resolution = resolution))[focus_ind(.G(), 'nodes')])
+  desc_enumeration(group)
+}
+#' @describeIn group_graph Group nodes according to the Leiden algorithm ([igraph::cluster_leiden()]) which is similar, but more efficient and provides higher quality results than `cluster_louvain()`
+#' @importFrom igraph membership cluster_leiden
+#' @importFrom rlang list2 inject :=
+#' @export
+group_leiden <- function(weights = NULL, resolution = 1, objective_function = 'CPM', beta = 0.01, label = NULL, n = 2, node_weights = NULL) {
+  expect_nodes()
+  graph <- .G()
+  weights <- enquo(weights)
+  weights <- eval_tidy(weights, .E()) %||% NA
+  # `resolution_parameter` may be renamed to `resolution` in future release of igraph
+  res_arg <- if ("resolution" %in% names(formals(igraph::cluster_leiden))) {
+    'resolution'
+  } else {
+    'resolution_parameter'
   }
-  group <- as.integer(membership(cluster_louvain(graph = .G(), weights = weights)))
+  nodes <- .N(focused = FALSE)
+  label <- enquo(label)
+  label <- eval_tidy(label, nodes)
+  node_weights <- enquo(node_weights)
+  node_weights <- eval_tidy(node_weights, nodes)
+  args <- list2(
+    graph = graph,
+    objective_function = objective_function,
+    weights = weights,
+    {{res_arg}} := resolution,
+    beta = beta,
+    initial_membership = label,
+    n_iterations = n,
+    vertex_weights = node_weights
+  )
+
+  group <- as.integer(membership(inject(cluster_leiden(!!!args)))[focus_ind(.G(), 'nodes')])
   desc_enumeration(group)
 }
 #' @describeIn group_graph Group nodes by optimising the moldularity score using [igraph::cluster_optimal()]
@@ -134,11 +182,8 @@ group_louvain <- function(weights = NULL) {
 group_optimal <- function(weights = NULL) {
   expect_nodes()
   weights <- enquo(weights)
-  weights <- eval_tidy(weights, .E())
-  if (is.null(weights)) {
-    weights <- NA
-  }
-  group <- as.integer(membership(cluster_optimal(graph = .G(), weights = weights)))
+  weights <- eval_tidy(weights, .E()) %||% NA
+  group <- as.integer(membership(cluster_optimal(graph = .G(), weights = weights))[focus_ind(.G(), 'nodes')])
   desc_enumeration(group)
 }
 #' @describeIn group_graph Group nodes using simulated annealing with [igraph::cluster_spinglass()]
@@ -147,24 +192,33 @@ group_optimal <- function(weights = NULL) {
 group_spinglass <- function(weights = NULL, ...) {
   expect_nodes()
   weights <- enquo(weights)
-  weights <- eval_tidy(weights, .E())
-  if (is.null(weights)) {
-    weights <- NA
-  }
-  group <- as.integer(membership(cluster_spinglass(graph = .G(), weights = weights, vertex = NULL, ...)))
+  weights <- eval_tidy(weights, .E()) %||% NA
+  group <- as.integer(membership(cluster_spinglass(graph = .G(), weights = weights, vertex = NULL, ...))[focus_ind(.G(), 'nodes')])
   desc_enumeration(group)
 }
 #' @describeIn group_graph Group nodes via short random walks using [igraph::cluster_walktrap()]
-#' @importFrom igraph membership cluster_walktrap
+#' @importFrom igraph membership cluster_walktrap cut_at
 #' @export
-group_walktrap <- function(weights = NULL, steps = 4) {
+group_walktrap <- function(weights = NULL, steps = 4, n_groups = NULL) {
   expect_nodes()
   weights <- enquo(weights)
-  weights <- eval_tidy(weights, .E())
-  if (is.null(weights)) {
-    weights <- NA
+  weights <- eval_tidy(weights, .E()) %||% NA
+  group <- cluster_walktrap(graph = .G(), weights = weights, steps = steps)
+  if (is.null(n_groups)) {
+    group <- membership(group)
+  } else {
+    group <- cut_at(group, no = n_groups)
   }
-  group <- as.integer(membership(cluster_walktrap(graph = .G(), weights = weights, steps = steps)))
+  group <- as.integer(group[focus_ind(.G(), 'nodes')])
+  desc_enumeration(group)
+}
+#' @describeIn group_graph Group nodes by simulating fluid interactions on the graph topology using [igraph::cluster_fluid_communities()]
+#' @importFrom igraph cluster_fluid_communities
+#' @export
+group_fluid <- function(n_groups = 2) {
+  expect_nodes()
+  graph <- .G()
+  group <- as.integer(membership(cluster_fluid_communities(graph, n_groups))[focus_ind(graph, 'nodes')])
   desc_enumeration(group)
 }
 #' @describeIn group_graph Group edges by their membership of the maximal binconnected components using [igraph::biconnected_components()]
@@ -175,7 +229,18 @@ group_biconnected_component <- function() {
   graph <- .G()
   comp <- biconnected_components(graph)
   ind <- lapply(comp$component_edges, as.integer)
-  group <- rep(seq_along(ind), lengths(ind))[order(unlist(ind))]
+  group <- rep(seq_along(ind), lengths(ind))[order(unlist(ind))][focus_ind(.G(), 'edges')]
+  desc_enumeration(group)
+}
+#' @describeIn group_graph Groups nodes by their color using [igraph::greedy_vertex_coloring()]. Be aware that this is not a clustering algorithm as coloring specifically provide a color to each node so that no neighbors have the same color
+#' @importFrom igraph greedy_vertex_coloring
+#' @export
+group_color <- function() {
+  expect_nodes()
+  graph <- .G()
+  group <- greedy_vertex_coloring(graph)
+
+  group <- as.integer(group[focus_ind(.G(), 'nodes')])
   desc_enumeration(group)
 }
 
